# The math of Llama

How to calculate the number of parameters in a Llama model.

The most relevant things in the model are the number of layers **N**, the number of dimensions for each token embedding **dim**, and the number of heads in a multi-head attention **h**. These mostly dictate how the model scales. The vocab size of all the models is 32.000 words/tokens.

## Llama 7B

```
N = 32 layers
dim = 4096
h = 32 heads

Embedding layer = vocab size * dim = 32.000 * 4096 = 131.072.000

Multi-head attention layer = h heads * 4 linear layers (3 for each input + 1 final concatenation layer) = 32 * 4 * 4096 * 4096 = 32 * 2.097.152 = 67.108.864
Feed forward layer = 3 linear layers * dim * (8 * dim / 3 *approx. to multiple of 256* ) = 3 * 4096 * 11.008 = 135.266.304
Norm layer = 1 * dim = 4096
1 block/transformer layer = multi-head attention + feed forward + 2 * norm = 67.108.864 + 135.266.304 + 2 * 4096 = 202.383.360
Final linear layer (output probabilities) = dim * vocab size = 4096 * 32.000 = 131.072.000

Whole model = input embedding layer + N layers + final norm layer + final linear layer = 131.072.000 + 32 * 202.383.360 + 4096 + 131.072.000 = 6.738.415.616 parameters
```

## Llama 13B

Similarly to Llama 7B, it is only a matter of plugging the numbers in:

```
N = 40 layers
dim = 5120
h = 40 heads

Embedding layer = vocab size * dim = 32.000 * 5120 = 163.840.000

Multi-head attention layer = h heads * 4 linear layers (3 for each input + 1 final concatenation layer) = 40 * 4 * 5120 * 5120 = 40 * 2.621.440 = 104.857.600
Feed forward layer = 3 linear layers * dim * (8 * dim / 3 *approx. to multiple of 256* ) = 3 * 5120 * 13.824 = 212.336.640
Norm layer = 1 * dim = 5120
1 block/transformer layer = multi-head attention + feed forward + 2 * norm = 104.857.600 + 212.336.640 + 2 * 5120 = 317.204.480
Final linear layer (output probabilities) = dim * vocab size = 5120 * 32.000 = 163.840.000

Whole model = input embedding layer + N layers + final norm layer + final linear layer = 163.840.000 + 40 * 317.204.480 + 5120 + 163.840.000 = 13.015.864.320 parameters
```

## Llama 70B

Llama 70B is a little bit of a different beast (pun intended). Instead of multi-head attention, it uses grouped-query attention. The math goes as follows: TODO(alexghergh)